<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
class PROTECTEDPHOTOS_CLASS_SelectedFriendListFormElement extends PROTECTEDPHOTOS_CLASS_FormElement
{
    private $selectedFriendList = array();

    private $selectedInput;

    public function __construct( $name, $albumId )
    {
        parent::__construct($name);

        $this->selectedInput = new HiddenField($this->getName());

        $friends = PROTECTEDPHOTOS_BOL_Service::getInstance()->getFriendIds($albumId);
        $displayNames = BOL_UserService::getInstance()->getDisplayNamesForList($friends);
        $avatars = BOL_AvatarService::getInstance()->getAvatarsUrlList($friends);

        foreach ( $friends as $friendId )
        {
            $this->selectedFriendList[$friendId] = array(
                'id' => $friendId,
                'displayName' => $displayNames[$friendId],
                'src' => $avatars[$friendId]
            );
        }

        OW::getLanguage()->addKeyForJs('protectedphotos', 'individual_friends_error');
    }

    public function setValue( $value )
    {
        $this->selectedInput->setValue($value);
    }

    public function getValue()
    {
        return $this->selectedInput->getValue();
    }

    public function addValidator( $validator )
    {
        $this->selectedInput->addValidator($validator);
    }

    public function getElementJs()
    {
        return $this->selectedInput->getElementJs();
    }

    public function getSelectedFriends()
    {
        return array_keys($this->selectedFriendList);
    }

    public function renderInput( $params = null )
    {
        $markup = '<div class="ow_pass_protected_selected_list ow_privacy_userlist_selected ow_right">
            {$input}
            <h3 class="ow_smallmargin">{$title}</h3>
            <div class="ow_privacy_userlist_items">
                <div class="ow_privacy_userlist_selected_content">
                    {$list}
                </div>
            </div>
            <div class="ow_hidden">
                <div class="ow_privacy_userlist_selected_item ow_border ow_bg_color ow_smallmargin clearfix">
                    <div class="ow_privacy_userlist_item_avatar">
                        <div class="ow_avatar">
                            <img style="max-width: 100%;" src="data:image/gif;base64,R0lGODlhAQABAAD/ACwAAAAAAQABAAACADs%3D">
                        </div>
                    </div>
                    <div class="ow_privacy_userlist_item_info">
                        <div class="ow_privacy_userlist_item_info_string ow_small">
                            <b></b>
                        </div>
                        <div class="ow_privacy_userlist_item_close ow_border"><a href="javascript://"></a></div>
                    </div>
                </div>
            </div>
        </div>';

        $objRef = $this;
        
        $friendsMarkup = array_reduce($this->selectedFriendList, function( $carry, $friend ) use ($objRef)
        {
            $carry .= $objRef->replace('<div class="ow_privacy_userlist_selected_item ow_border ow_bg_color ow_smallmargin clearfix" data-id="{$id}" data-display-name="{$displayName}" data-src="{$src}">
                <div class="ow_privacy_userlist_item_avatar">
                    <div class="ow_avatar">
                        <img style="max-width: 100%;" title="{$displayName}" alt="{$displayName}" src="{$src}">
                    </div>
                </div>
                <div class="ow_privacy_userlist_item_info">
                    <div class="ow_privacy_userlist_item_info_string ow_small">
                        <b>{$displayName}</b>
                    </div>
                    <div class="ow_privacy_userlist_item_close ow_border"><a href="javascript://"></a></div>
                </div>
            </div>', $friend);

            return $carry;
        }, '');

        return $this->replace($markup, array(
            'input' => $this->selectedInput->renderInput(),
            'title' => OW::getLanguage()->text('protectedphotos', 'selected_title'),
            'list' => $friendsMarkup
        ));
    }
}
